package	jp.co.sds_corp.tool.dba;

public class ColumnInfo
{
	String	name;
	String	type;
	String	jname;
	int		nSize = 0;
	int		nPrec = 0;
	int		nScale = 0;
	boolean	bKey = false;
	String	desc;
	int		nIdx;
	int		methodType;
	int		columnType;
	public static final int NOCHANGE = 0;
	public static final int LOWER = 1;
	public static final int UPPER = 2;

	public ColumnInfo( String[] args )
		throws IllegalArgumentException
	{
		String buf = "";
		for ( int i = 0; i < args.length; i++ ) {
			buf += args[i] + "|";
		}
// System.out.println( args.length + ")" + buf );
		switch ( args.length ) {
		case 4:
			setJName( args[1] );
			setName( args[2] );
			setFullType( args[3] );
			break;
		case 5:
			setJName( args[1] );
			setName( args[2] );
			setFullType( args[3] );
			setKey( args[4].equals( "○" ) );
			break;
		case 6:
			setJName( args[1] );
			setName( args[2] );
			setFullType( args[3] );
			setKey( args[4].equals( "○" ) );
			setDesc( args[5] );
			break;
		case 8:
			setJName( args[1] );
			setName( args[2] );
			setFullType( args[3] );
			if ( args[4].length() > 0 ) {
				setSize( Integer.valueOf( args[4] ).intValue());
			}
			if ( hasPrecision()) {
				setPrecision( getSize());
				if ( args[5].length() > 0 ) {
					setScale( Integer.valueOf( args[5] ).intValue());
				}
			}
			setKey( args[6] != null && args[6].length() != 0 );
			if ( args.length == 8 ) {
				setDesc( args[7] );
			}
			break;
		default:
			throw( new IllegalArgumentException( "Illegal line data [" + buf + "]" ));
		}
	}

	public void setName( String var ) {
		this.name = var.trim();
	}
	public String getName() {
		String ret = this.name;
		if ( columnType == LOWER ) {
			ret = this.name.toLowerCase();
		} else
		if ( columnType == UPPER ) {
			ret = this.name.toUpperCase();
		}
		return ret;
	}

	private String getType( String var ) {
		var = var.replaceAll( " ", "" );
		int sta = var.indexOf( "(" );
		if ( sta < 0 ) {
			return var;
		}
		return var.substring( 0, sta );
	}
	private String getLength( String var ) {
		var = var.replaceAll( " ", "" );
		int sta = var.indexOf( "(" );
		int end = var.indexOf( ")" );
		if ( sta < 0 || end < 0 ) {
			return null;
		}
		return var.substring( sta+1, end );
	}
	private int getSize( String var ) {
		int nIdx = var.indexOf( "," );
		if ( nIdx >= 0 ) {
			var = var.substring( 0, nIdx );
		}
		return Integer.valueOf( var ).intValue();
	}
	private int getScale( String var ) {
		int nIdx = var.indexOf( "," );
		if ( nIdx < 0 ) {
			return 0;
		}
		var = var.substring( nIdx + 1 );
		return Integer.valueOf( var ).intValue();
	}

	public String getFullType() {
		String ret = getType();
		if ( hasPrecision()) {
			if ( getScale() > 0 ) {
				ret = getType() + "(" + getPrecision() + "," + getScale() + ")";
			} else {
				ret = getType() + "(" + getPrecision() + ")";
			}
		} else {
			if ( getSize() > 0 ) {
				ret = getType() + "(" + getSize() + ")";
			}
		}
		return ret;
	}
	public void setFullType( String var ) {
		String type = getType( var );
		setType( type );
		String len = getLength( var );
		if ( len != null ) {
			setSize( getSize( len ));
			if ( hasPrecision()) {
				setPrecision( getSize());
				setScale( getScale( len ));
			}
		}
	}
	public void setType( String var ) {
		this.type = var.toLowerCase();
	}
	public String getType() {
		return this.type;
	}

	public void setJName( String var ) {
		this.jname = var;
	}
	public String getJName() {
		return this.jname;
	}

	public void setKey( boolean var ) {
		this.bKey = var;
	}
	public boolean isKey() {
		return this.bKey;
	}

	public void setDesc( String var ) {
		this.desc = var;
	}
	public String getDesc() {
		return this.desc;
	}

	public void setIndex( int var ) {
		this.nIdx = var;
	}
	public int getIndex() {
		return this.nIdx;
	}

	public void setSize( int var ) {
		this.nSize = var;
	}
	public int getSize() {
		return this.nSize;
	}

	public void setPrecision( int var ) {
		this.nPrec = var;
	}
	public int getPrecision() {
		return this.nPrec;
	}

	public void setScale( int var ) {
		this.nScale = var;
	}
	public int getScale() {
		return this.nScale;
	}

	public static String[] dbTypes = {
		"char", "varchar2", "varchar", "nchar", "text",
		"int", "short", "float", "double", "tinyint",
		"decimal", "numeric", "number",
		"timestamp", "datetime", "date"
	};
	public static String[] sqlTypes = {
		"CHAR", "VARCHAR", "VARCHAR", "NCHAR", "LONGVARCHAR",
		"INTEGER", "SMALLINT", "DOUBLE", "DOUBLE", "TINYINT",
		"DECIMAL", "DECIMAL", "DECIMAL",
		"TIMESTAMP", "TIMESTAMP", "TIMESTAMP"
	};
	public static String[] javaTypes = {
		"String", "String", "String", "String", "String",
		"Integer", "Short", "Double", "Double", "Integer",
		"BigDecimal", "BigDecimal", "BigDecimal",
		"Timestamp", "Timestamp", "Timestamp"
	};

	public String getSqlType() {
		String type = null;
		for ( int i = 0; i < dbTypes.length; i++ ) {
			if ( getType().equals( dbTypes[i] )) {
				type = "Types." + sqlTypes[i];
				break;
			}
		}
		if ( type == null ) {
			throw( new RuntimeException( "type[" + getType() + "] not found." ));
		}
		return type;
	}

	public String getJavaType() {
		String type = null;
		for ( int i = 0; i < dbTypes.length; i++ ) {
			if ( getType().equals( dbTypes[i] )) {
				type = javaTypes[i];
				break;
			}
		}
		if ( type == null ) {
			throw( new RuntimeException( "type[" + getType() + "] not found." ));
		}
		return type;
	}

	public static String[] numericTypes = {
		"Integer", "Short", "Double", "BigDecimal"
	};
	private boolean isNumeric() {
		String type = getJavaType();
		for ( int i = 0; i < numericTypes.length; i++ ) {
			if ( type.equals( numericTypes[i] )) {
				return true;
			}
		}
		return false;
	}

	public static String[] precTypes = {
		"Types.CHAR", "Types.VARCHAR", "Types.DECIMAL"
	};
	private boolean hasPrecision() {
		String type = getSqlType();
		for ( int i = 0; i < precTypes.length; i++ ) {
			if ( type.equals( precTypes[i] ) ) {
				return true;
			}
		}
		return false;
	}

	public String getMethodName() {
		String rest = getName().substring( 1 );
		if ( getMethodType() == LOWER ) {
			rest = rest.toLowerCase();
		} else
		if ( getMethodType() == UPPER ) {
			rest = rest.toUpperCase();
		}

		return getName().substring( 0, 1 ).toUpperCase() + rest;
	}

	public void setMethodType( int var ) {
		this.methodType = var;
	}
	public int getMethodType() {
		return this.methodType;
	}

	public void setColumnType( int var ) {
		this.columnType = var;
	}
	public int getColumnType() {
		return this.columnType;
	}

	public String toString() {
		String ret = "name=" + getName() + " type=" + getType()
			+ " jname=" + getJName() + " key=" + isKey()
			+ " size=" + getSize()
			+ " scale=" + getScale() + " prec=" + getPrecision()
			+ " sql=" + getSqlType() + " java=" + getJavaType()
			+ " method=" + getMethodName();
		return ret;
	}

}
