package	jp.co.sds_corp.tool.dba;

import java.util.Vector;

public class TableLayout
{
	String	file_name = null;
	Vector<ColumnInfo>	vecClm = new Vector<ColumnInfo>();
	public static final int		DB_SYBASE = 1;
	public static final int		DB_ORACLE = 2;

	int		nDbType = DB_SYBASE;
	String	useTool;
	String	pkg;
	String	util_pkg;
	String	audit_column;
	String	encode;
	int		methodType;
	int		columnType;
	boolean debug = false;

	public TableLayout( String file_name )
		throws Exception
	{
		this.file_name = file_name;
		String type = System.getProperty( "table.dbtype" );
		if ( type != null ) {
			if ( type.equals( "Oracle" )) {
				nDbType = DB_ORACLE;
			}
		}
		useTool = System.getProperty( "table.usesqltool", "false" );
		pkg = System.getProperty( "table.package", "table" );
		util_pkg = System.getProperty( "table.util.package", "jp.co.sds_corp.util" );
		audit_column = System.getProperty( "table.audit", "audit_datetime" );
		String method = System.getProperty( "table.method", "" );
		if ( method.equals( "lower" )) {
			methodType = ColumnInfo.LOWER;
		} else
		if ( method.equals( "upper" )) {
			methodType = ColumnInfo.UPPER;
		}
		String col = System.getProperty( "table.column", "" );
		if ( col.equals( "lower" )) {
			columnType = ColumnInfo.LOWER;
		} else
		if ( col.equals( "upper" )) {
			columnType = ColumnInfo.UPPER;
		}
		encode = System.getProperty( "table.encode" );
		String dbg = System.getProperty( "table.debug", "false" );
		debug = dbg.equals( "true" );
	}

	public void createTable() throws Exception {
		TableGenerator tg;
		switch ( nDbType ) {
		case DB_SYBASE:
			tg = new TableSybase( file_name );
			break;
		case DB_ORACLE:
			tg = new TableOracle( file_name );
			break;
		default:
			throw( new IllegalArgumentException( "Huh? " + nDbType ));
		}
		if ( useTool != null && useTool.equals( "true" )) {
			tg.setUseSqlTool( true );
		}
		tg.setEncode( encode );
		tg.setDebug( debug );
		tg.create();
	}

	public void createJava() throws Exception {
		TableGenerator tg;
		String[] arr = audit_column.split( "," );
		tg = new TableJava( file_name, pkg, util_pkg, arr, nDbType );
		tg.setMethodType( methodType );
		tg.setColumnType( columnType );
		tg.setEncode( encode );
		tg.create();
	}

	public static void main( String[] args ) {
		try {
			for ( int i = 0; i < args.length; i++ ) {
				System.out.println( args[i] );
				TableLayout tbl = new TableLayout( args[i] );
				tbl.createTable();
				tbl.createJava();
			}
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

}

