package	jp.co.sds_corp.tool.docs;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.Comparator;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import jp.co.sds_corp.util.common.CmnString;

public class DocsConverter
{
	class History
	{
		int lineNo;
		String	date;
		String	ver;
		Vector<String>	vecDesc = new Vector<String>();

		History( int lineNo, String date, String ver ) {
			this.lineNo = lineNo;
			this.date = date;
			this.ver = ver;
		}

		void addDesc( String line ) {
			vecDesc.add( line );
		}

		String getHtml() {
			String ret = "<LI><STRONG>" + date + "</STRONG>: "
				+ "<A NAME=\"ver" + ver + "\">"
					+ pkg + " ver " + ver + "</A>\n";
			if ( vecDesc.size() > 0 ) {
				ret += "<UL>\n";
				for ( int i = 0; i < vecDesc.size(); i++ ) {
					ret += "\t<LI>" + vecDesc.get( i ) + "\n";
				}
				ret += "</UL>\n";
			}
			ret += "<BR>\n";
			return ret;
		}
	}

	String	histFile = "CHANGES";
	String	tempFile = "index.temp";
	String	outFile = "index.html";
	String	pattern = "<$$$ History $$$>";
	StringBuffer	template = new StringBuffer();
	String	pkg;
	TreeMap<String,History>	mapHist = new TreeMap<String,History>(
		new Comparator<String>()
		{
			public int compare( String s1, String s2 ) {
				if ( s1.compareTo( s2 ) < 0 ) {
					return 1;
				} else if ( s1.compareTo( s2 ) > 0 ) {
					return -1;
				} else {
					return 0;
				}
			}
			public boolean equals( Object obj ) {
				return false;
			}
		}
	);

	public DocsConverter() {
	}

	public DocsConverter( String histFile, String tempFile, String outFile ) {
		this.histFile = histFile;
		this.tempFile = tempFile;
		this.outFile = outFile;
	}

	protected void readTemplate( String file ) throws Exception {
		BufferedReader br = new BufferedReader( new FileReader( file ) );
		String  line;
		for ( ;; ) {
			line = br.readLine();
			if ( line == null ) {
				break;
			}
			template.append( line + "\n" );
		}
		br.close();
	}

	public void convert() throws Exception {
		readTemplate( tempFile );

		BufferedReader br = new BufferedReader( new FileReader( histFile ) );
		int     nLine = 0;
		String  line;
		History hist = null;
		for ( ;; ) {
			line = br.readLine();
			if ( line == null ) {
				break;
			}
			nLine++;
			// 空行は読み飛ばし
			if ( line.length() == 0 ) {
				continue;
			}
			String[] buf = CmnString.getStringToken( line, "\t", false );
			if ( line.charAt( 0 ) == '\t' || buf.length == 1) {
				if ( hist == null ) {
					System.err.println( "Ignore line [" + nLine + "]" );
					continue;
				}
				if ( line.charAt( 0 ) != '\t' ) {
					System.err.println("illegal format. line=" + nLine + " " + line);
				}
				hist.addDesc( line.substring( 1 ) );
			} else {
				hist = new History( nLine, buf[0], buf[1] );
				mapHist.put( hist.date, hist );
			}
		}

		StringBuffer rep = new StringBuffer();
		rep.append( "<UL>" );
		Iterator<String> itr = mapHist.keySet().iterator();
		while ( itr.hasNext()) {
			String d = (String )itr.next();
			History h = (History )mapHist.get( d );
			rep.append( h.getHtml() );
		}
		rep.append( "</UL>" );

		//PrintWriter out = new PrintWriter( System.out );
		PrintWriter out = new PrintWriter( new FileWriter( outFile ));
		int nPos = template.indexOf( pattern );
		if ( nPos < 0 ) {
			System.err.println( "[" + pattern + "] not found in template file");
		} else {
			template.replace( nPos, nPos + pattern.length(), rep.toString() );
		}
		out.print( template.toString() );
		out.flush();
		out.close();
		br.close();
	}

	public static void main( String[] args ) {
		try {
			if ( args.length < 1 ) {
				throw( new Exception( "require package name" ));
			}
			DocsConverter	obj;
			if ( args.length == 1 ) {
				obj = new DocsConverter();
			} else {
				obj = new DocsConverter( args[1], args[2], args[3] );
			}
			obj.pkg = args[0];
			obj.convert();
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}
}
